# include "OccamOut.h"
# include "yyOccamOut.w"
# include "System.h"
# include <stdio.h>
# include "Tree.h"

# define yyInline
# ifndef NULL
# define NULL 0L
# endif
# ifndef false
# define false 0
# endif
# ifndef true
# define true 1
# endif

# ifdef yyInline
# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) \
  if ((ptr = (tree) free) >= (tree) max) ptr = alloc (); \
  free += nodesize [kind]; \
  ptr->yyHead.yyMark = 0; \
  ptr->Kind = kind;
# else
# define yyALLOC(tree, free, max, alloc, nodesize, make, ptr, kind) ptr = make (kind);
# endif

# define yyWrite(s) (void) fputs (s, yyf)
# define yyWriteNl (void) fputc ('\n', yyf)

/* line 29 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */

  #include <stdio.h>
  #include <stdlib.h>
  #include <TypeChk.h>

  /*{{{  statics*/
  static indent = 0;
  static col = 0;   
  static bool NL = false;  
  extern FILE* yyf;
  static void yyAbort(char *);
  
  /*}}}*/
  /*{{{  defines*/
  #undef yyWrite
  
  static void Indent()     { if (!NL) fputc('\n',yyf); NL=true; indent+=2; }
  static void Outdent()    { if (!NL) fputc('\n',yyf); NL=true; indent-=2; }
  static void NewLine()    { if (!NL) fputc('\n',yyf); NL=true;}
  static void DoNL()       { if (NL==true) { int i; NL = false; for(i=0;i<indent;i++) fputc(' ',yyf); col=indent;} }
  
  #define WriteInt(a)  {DoNL(); col+=fprintf(yyf,"%d",(int)a);}
  #define WriteReal(a) {DoNL(); col+=fprintf(yyf,"%*G",a.RNumber.p,a.RNumber.n);}
  #define WriteByte(a) {DoNL(); col+=fprintf(yyf,"\'*#%02X\'",(unsigned)a);}
  #define WriteBool(a) {DoNL(); col+=fprintf(yyf,"%s",(a)?"TRUE":"FALSE");}
  #define WriteStr(a)  {DoNL(); col+=fprintf(yyf,"%s",a);}
  #define WriteChar(a)  {DoNL(); col+=1; fputc(a,yyf);}
  #define WriteId(a)   { DoNL(); WriteIdent(yyf,a); }
  
  #define SetIndent(a)   (a=indent,indent=col)
  #define ClearIndent(a) (indent=a)
  #define COND_Write(b,s) { if (b) yyWrite(s); }
  
  #define SizeInt      sizeof(int)
  
  #define yyWrite(s)   {DoNL(); col+=fprintf(yyf,"%s",s);}
  
  static void WriteSym(tSymbol s)   
  {
    DoNL(); 
    if (Symbol(s).Id==NoIdent) 
      col+=fprintf(yyf,"tmp.%d",s); 
    else 
    { 
      tStringRef id; 
      id=GetStringRef(Symbol(s).Id); 
      col+=LengthSt(id);
      WriteIdent(yyf,Symbol(s).Id);
    }
  }  
  
  #define WarnNotImp(P,S)    ErrorMessageI(xxNotImplemented,xxWarning,P,xxString,S)
  
  #define IsPrintable(c) (((c)>=' ') && ((c)<='~'))
  
  
  /*}}}*/
  /*{{{  functions*/
  /*{{{  static void WriteOccStr(char *str,int len)*/
  static void WriteOccStr(char *str,int len)
  {
    int i;
  
    for (i=0;i<len;i++)
    {
      char c = str[i] ;
    
      if (c == '*' || c == '"' || c == '\'')
      {
        fprintf(yyf,"*%c",c) ;
        col+=2;
      }
      else if (IsPrintable(c))
      {
        fputc(c,yyf) ;
        col++ ;
      }
      else if (c == 13)
      {
        fprintf(yyf,"*c") ;
        col+=2;
      }
      else if (c == 10)
      {
        fprintf(yyf,"*n") ;
        col+=2;
      }
      else if (c == '\t')
      {
        fprintf(yyf,"*t") ;
        col+=2;
      }
      else
      {
        fprintf(yyf,"*#%02x",c) ;
        col+=4;
      }
    }
  }
  
  /*}}}*/
  /*{{{  void WriteStrRef(tStringRef r)*/
  void WriteStrRef(tStringRef r)
  {
    char *temp ;
    int len = LengthSt(r) ;
  
    DoNL() ;
  
    if ((temp = malloc(len)) == NULL)
      yyAbort("Out of heap") ;
    else
      {
        int i;
        
        StGetString (r,temp) ;
        WriteOccStr(temp,len);
        free(temp) ;
      }
  }
  /*}}}*/
  /*{{{  void WritetInt(tInt val,bool int64)*/
  static void WritetInt(tInt val,bool int64)
  {
    char str[20];
    int len=0;
    bool neg=(val < 0);
    tInt value=(neg)?-val:val;
    bool hex=(val > 1000000) || (val < -1000000);
  
    DoNL() ;
  
    if (hex)
    {
      int digits = (int64) ? 16 : 8;
      char hexdigits[] = "0123456789ABCDEF";
  
      while(len < digits) { str[len++] = hexdigits[val & 15]; val = val >> 4; }
      while(len>1 && str[len-1]=='0') --len;
  
      fprintf(yyf,"#");
    }
    else
    {
      while(len==0 || value > 0) { str[len++]='0'+(value % 10); value = value / 10; }
      if (neg) fputc('-',yyf);
    }
  
    {
      int i = 0;
      int j=len-1;
      char c;
  
      while(i < j) { c = str[i]; str[i++] = str[j]; str[j--] = c; }
    }
    
    str[len] = 0;
    fprintf(yyf,"%s",str);
  }
    
    
  
  /*}}}*/
  /*{{{  void OutputOccam(char *name, tTree Root)*/
  void OutputOccam(char *name, tTree Root)
  {
    FILE *savef = yyf;
  
    yyf = stdout;
    if (strcmp(name,"-") != 0)
      if ((yyf=fopen(name,"w"))==NULL) MessageI("Cannot open file ",xxFatal,NoPosition,xxString,name);
  
    Code(Root);
    if (yyf != stdout) fclose(yyf);
  
    yyf = savef;
  }
  /*}}}*/
  /*}}}*/


static void yyExit () { Exit (1); }

void (* OccamOut_Exit) () = yyExit;

extern FILE *yyf;

static void yyAbort
# ifdef __cplusplus
 (char * yyFunction)
# else
 (yyFunction) char * yyFunction;
# endif
{
 (void) fprintf (stderr, "Error: module OccamOut, routine %s failed\n", yyFunction);
 OccamOut_Exit ();
}

void Code ARGS((tTree yyP1));
static void CodeExpr ARGS((tTree yyP2));
static void CodeConst ARGS((tTree yyP4, tValue yyP3));
static void CodeBy ARGS((tTree yyP5));
static void CodeOp ARGS((tTree yyP6));
void CodeType ARGS((tTree yyP7));
static void CodeConstArray ARGS((tTree yyP9, tConstArray yyP8));
static void SemiSep ARGS((tTree yyP10));

void Code
# if defined __STDC__ | defined __cplusplus
(register tTree yyP1)
# else
(yyP1)
 register tTree yyP1;
# endif
{

  switch (yyP1->Kind) {
  case kASpec:
/* line 214 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 214 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ASpec.Spec);
/* line 214 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ASpec.Next);
  }
   return;

  case kNoSpec:
/* line 215 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   return;

  case kIs:
/* line 217 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 217 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP1->Is.Type);
/* line 217 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->Is.Sym);
/* line 217 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" IS ");
/* line 217 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->Is.Elemental);
/* line 217 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (":");
/* line 217 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kVal:
/* line 218 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 218 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("VAL ");
/* line 218 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP1->Val.Type);
/* line 218 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->Val.Sym);
/* line 218 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" IS ");
/* line 218 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->Val.Expr);
/* line 218 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (":");
/* line 218 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kDefProc:
/* line 220 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 220 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("PROC ");
/* line 220 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->DefProc.Sym);
/* line 220 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(");
/* line 220 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->DefProc.Formals);
/* line 220 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (")");
/* line 220 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 220 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->DefProc.Process);
/* line 220 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
/* line 220 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (":");
/* line 220 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kDecl:
/* line 222 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 222 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP1->Decl.Type);
/* line 222 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Decl.IdList);
/* line 222 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (":");
/* line 222 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kDefSeqPro:
/* line 223 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 223 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("PROTOCOL ");
/* line 223 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->DefSeqPro.Sym);
/* line 223 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" IS ");
/* line 223 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->DefSeqPro.SeqProList);
/* line 223 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (":");
/* line 223 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kDefTagPro:
/* line 224 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 224 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("PROTOCOL ");
/* line 224 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->DefTagPro.Sym);
/* line 224 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 224 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("CASE ");
/* line 224 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 224 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->DefTagPro.TagProList);
/* line 224 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
/* line 224 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
/* line 224 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (":");
/* line 224 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kDefFunc:
/* line 226 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 226 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP1->DefFunc.TypeList);
/* line 226 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("FUNCTION ");
/* line 226 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->DefFunc.Sym);
/* line 226 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(");
/* line 226 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->DefFunc.Formals);
/* line 226 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (")");
/* line 226 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 227 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->DefFunc.Valof->Valof.SpecList);
/* line 227 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("VALOF");
/* line 227 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 227 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->DefFunc.Valof->Valof.Process);
/* line 227 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("RESULT ");
/* line 227 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->DefFunc.Valof->Valof.ExprList);
/* line 227 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
/* line 227 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
/* line 227 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (":");
/* line 227 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kDefIsFunc:
/* line 228 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 228 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP1->DefIsFunc.TypeList);
/* line 228 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("FUNCTION ");
/* line 228 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->DefIsFunc.Sym);
/* line 228 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(");
/* line 228 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->DefIsFunc.Formals);
/* line 228 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (")");
/* line 228 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" IS ");
/* line 228 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->DefIsFunc.ExprList);
/* line 228 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (":");
/* line 228 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kRetypes:
/* line 230 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 230 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP1->Retypes.Type);
/* line 230 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->Retypes.Sym);
/* line 230 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" RETYPES ");
/* line 230 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->Retypes.Elemental);
/* line 230 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" :");
/* line 230 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kValRetypes:
/* line 231 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 231 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("VAL ");
/* line 231 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP1->ValRetypes.Type);
/* line 231 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->ValRetypes.Sym);
/* line 231 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" RETYPES ");
/* line 231 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->ValRetypes.Expr);
/* line 231 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" :");
/* line 231 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kNoFormal:
/* line 233 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   return;

  case kAFormal:
  if (yyP1->AFormal.Next->Kind == kNoFormal) {
/* line 234 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 234 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeBy (yyP1->AFormal.By);
/* line 234 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP1->AFormal.Type);
/* line 234 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->AFormal.Sym);
  }
   return;

  }
/* line 235 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 235 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeBy (yyP1->AFormal.By);
/* line 235 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP1->AFormal.Type);
/* line 235 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->AFormal.Sym);
/* line 235 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (", ");
/* line 235 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AFormal.Next);
  }
   return;

  case kReplicator:
/* line 237 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 237 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->Replicator.Sym);
/* line 237 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" = ");
/* line 237 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->Replicator.From);
/* line 237 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" FOR ");
/* line 237 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->Replicator.For);
  }
   return;

  case kNoTagPro:
/* line 240 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   return;

  case kATagPro:
  if (yyP1->ATagPro.SeqProList->Kind == kNoSeqPro) {
/* line 241 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 241 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->ATagPro.Sym);
/* line 241 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
/* line 241 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ATagPro.Next);
  }
   return;

  }
/* line 242 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 242 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->ATagPro.Sym);
/* line 242 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("; ");
/* line 242 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ATagPro.SeqProList);
/* line 242 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
/* line 242 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ATagPro.Next);
  }
   return;

  case kAnId:
  if (yyP1->AnId.Next->Kind == kNoId) {
/* line 244 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 244 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->AnId.Sym);
  }
   return;

  }
/* line 245 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 245 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->AnId.Sym);
/* line 245 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (",");
/* line 245 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AnId.Next);
  }
   return;

  case kStart:
/* line 247 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 247 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Start.Process);
/* line 247 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
/* line 247 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("");
  }
   return;

  case kAny:
/* line 249 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 249 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("ANY ");
  }
   return;

  case kNamed:
/* line 250 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 250 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->Named.Sym);
/* line 250 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" ");
  }
   return;

  case kSimple:
/* line 251 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 251 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP1->Simple.Type);
  }
   return;

  case kCounted:
/* line 252 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 252 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP1->Counted.Length);
/* line 252 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (":: ");
/* line 252 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP1->Counted.Type);
  }
   return;

  case kASeqPro:
  if (yyP1->ASeqPro.Next->Kind == kNoSeqPro) {
/* line 254 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 254 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ASeqPro.Protocol);
  }
   return;

  }
/* line 255 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 255 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ASeqPro.Protocol);
/* line 255 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("; ");
/* line 255 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ASeqPro.Next);
  }
   return;

  case kAnExpr:
  if (yyP1->AnExpr.Next->Kind == kNoExpr) {
/* line 257 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 257 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->AnExpr.Expr);
  }
   return;

  }
/* line 258 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 258 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->AnExpr.Expr);
/* line 258 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (", ");
/* line 258 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AnExpr.Next);
  }
   return;

  case kNoCase:
/* line 260 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   return;

  case kACase:
/* line 261 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 261 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ACase.SpecList);
/* line 261 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   SemiSep (yyP1->ACase.ExprList);
/* line 261 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 261 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ACase.Process);
/* line 261 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
/* line 261 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ACase.Next);
  }
   return;

  case kAssign:
/* line 263 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 263 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Assign.SpecList);
/* line 263 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Assign.Lval);
/* line 263 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" := ");
/* line 263 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Assign.Rval);
/* line 263 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kAlt:
/* line 265 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 265 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Alt.SpecList);
/* line 265 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   COND_Write (yyP1->Alt.Pri, "PRI ");
/* line 265 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("ALT");
/* line 265 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 265 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Alt.AltList);
/* line 265 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kAltRep:
/* line 266 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 266 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AltRep.SpecList);
/* line 266 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   COND_Write (yyP1->AltRep.Pri, "PRI ");
/* line 266 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("ALT");
/* line 266 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AltRep.Replicator);
/* line 266 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 266 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AltRep.AltList);
/* line 266 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kInput:
/* line 268 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 268 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Input.SpecList);
/* line 268 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->Input.Element);
/* line 268 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" ? ");
/* line 268 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   SemiSep (yyP1->Input.ExprList);
/* line 268 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kOutput:
/* line 269 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 269 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Output.SpecList);
/* line 269 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->Output.Element);
/* line 269 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" ! ");
/* line 269 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   SemiSep (yyP1->Output.ExprList);
/* line 269 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kCaseIn:
/* line 270 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 270 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->CaseIn.SpecList);
/* line 270 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->CaseIn.Element);
/* line 270 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" ? ");
/* line 270 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("CASE");
/* line 270 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 270 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->CaseIn.CaseList);
/* line 270 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kAfter:
/* line 271 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 271 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->After.SpecList);
/* line 271 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->After.Element);
/* line 271 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" ? ");
/* line 271 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("AFTER ");
/* line 271 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->After.Expr);
/* line 271 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kSeq:
/* line 272 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 272 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Seq.SpecList);
/* line 272 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("SEQ");
/* line 272 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 272 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Seq.ProcList);
/* line 272 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kSeqRep:
/* line 273 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 273 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->SeqRep.SpecList);
/* line 273 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("SEQ ");
/* line 273 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->SeqRep.Replicator);
/* line 273 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 273 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->SeqRep.Process);
/* line 273 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kPar:
/* line 275 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 275 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Par.SpecList);
/* line 275 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   COND_Write (yyP1->Par.PType == xxPriPar, "PRI ");
/* line 275 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   COND_Write (yyP1->Par.PType == xxPlacedPar, "PLACED ");
/* line 275 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("PAR");
/* line 275 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 275 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Par.ProcList);
/* line 275 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kParRep:
/* line 276 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 276 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ParRep.SpecList);
/* line 276 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   COND_Write (yyP1->ParRep.PType == xxPriPar, "PRI ");
/* line 276 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   COND_Write (yyP1->ParRep.PType == xxPlacedPar, "PLACED ");
/* line 276 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("PAR ");
/* line 276 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ParRep.Replicator);
/* line 276 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 276 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ParRep.Process);
/* line 276 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kProcessor:
/* line 278 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 278 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Processor.SpecList);
/* line 278 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("PROCESSOR ");
/* line 278 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->Processor.Expr);
/* line 278 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" ");
/* line 278 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteId (yyP1->Processor.Name);
/* line 278 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
/* line 278 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 278 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Processor.Process);
/* line 278 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kIf:
/* line 280 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 280 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->If.SpecList);
/* line 280 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("IF");
/* line 280 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 280 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->If.ChoiceList);
/* line 280 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kIfRep:
/* line 281 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 281 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->IfRep.SpecList);
/* line 281 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("IF ");
/* line 281 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->IfRep.Replicator);
/* line 281 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 281 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->IfRep.ChoiceList);
/* line 281 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kCase:
/* line 282 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 282 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Case.SpecList);
/* line 282 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("CASE ");
/* line 282 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Case.Selector);
/* line 282 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 282 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Case.ChoiceList);
/* line 282 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kWhile:
/* line 283 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 283 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->While.SpecList);
/* line 283 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("WHILE ");
/* line 283 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->While.Expr);
/* line 283 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 283 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->While.Process);
/* line 283 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kSkip:
/* line 284 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 284 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Skip.SpecList);
/* line 284 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("SKIP");
/* line 284 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kStop:
/* line 285 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 285 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Stop.SpecList);
/* line 285 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("STOP");
/* line 285 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kCCode:
/* line 286 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 286 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->CCode.SpecList);
/* line 286 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("SKIP -- ");
/* line 286 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteStrRef (yyP1->CCode.Code);
/* line 286 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kInstance:
/* line 287 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 287 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Instance.SpecList);
/* line 287 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP1->Instance.Sym);
/* line 287 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(");
/* line 287 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->Instance.ExprList);
/* line 287 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (")");
/* line 287 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

  case kNullProcess:
/* line 288 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 288 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->NullProcess.SpecList);
  }
   return;

  case kNoAlt:
/* line 290 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   return;

  case kAnAlt:
/* line 291 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 291 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AnAlt.SpecList);
/* line 291 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AnAlt.Guard);
/* line 291 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AnAlt.Next);
  }
   return;

  case kANewAlt:
/* line 292 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 292 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ANewAlt.SpecList);
/* line 292 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ANewAlt.AltType);
/* line 292 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ANewAlt.Next);
  }
   return;

  case kAltInput:
  if (yyP1->AltInput.Expr->Kind == kConst) {
/* line 294 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 294 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   if (! ((yyP1->AltInput.Expr->Const.Value . Number == true))) goto yyL57;
  {
/* line 294 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AltInput.Input);
/* line 294 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 294 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AltInput.Process);
/* line 294 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
  }
   return;
yyL57:;

  }
/* line 295 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 295 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->AltInput.Expr);
/* line 295 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" & ");
/* line 295 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AltInput.Input);
/* line 295 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 295 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AltInput.Process);
/* line 295 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kAltTimer:
  if (yyP1->AltTimer.Expr->Kind == kConst) {
/* line 296 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 296 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   if (! ((yyP1->AltTimer.Expr->Const.Value . Number == true))) goto yyL59;
  {
/* line 296 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AltTimer.After);
/* line 296 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 296 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AltTimer.Process);
/* line 296 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
  }
   return;
yyL59:;

  }
/* line 297 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 297 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->AltTimer.Expr);
/* line 297 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" & ");
/* line 297 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AltTimer.After);
/* line 297 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 297 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AltTimer.Process);
/* line 297 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kAltCaseIn:
  if (yyP1->AltCaseIn.Expr->Kind == kConst) {
/* line 298 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 298 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   if (! ((yyP1->AltCaseIn.Expr->Const.Value . Number == true))) goto yyL61;
  {
/* line 298 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AltCaseIn.CaseIn);
  }
  }
   return;
yyL61:;

  }
/* line 299 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 299 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->AltCaseIn.Expr);
/* line 299 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" & ");
/* line 299 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AltCaseIn.CaseIn);
  }
   return;

  case kNoInput:
/* line 300 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 300 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1->NoInput.Expr);
/* line 300 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" & SKIP ");
/* line 300 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 300 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->NoInput.Process);
/* line 300 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
  }
   return;

  case kNoChoice:
/* line 302 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   return;

  case kAChoice:
/* line 303 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 303 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AChoice.SpecList);
/* line 303 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AChoice.ExprList);
/* line 303 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 303 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AChoice.Process);
/* line 303 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
/* line 303 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AChoice.Next);
  }
   return;

  case kANewIf:
/* line 304 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 304 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ANewIf.SpecList);
/* line 304 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ANewIf.IfType);
/* line 304 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->ANewIf.Next);
  }
   return;

  case kNoProcess:
/* line 306 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   return;

  case kAProcess:
/* line 307 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 307 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AProcess.Process);
/* line 307 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP1->AProcess.Next);
  }
   return;

  case kPrototypeFunc:
/* line 309 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   return;

  case kPrototypeProc:
/* line 310 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   return;

  case kParFrame:
/* line 312 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   return;

  case kCallFrame:
/* line 313 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   return;

  }

/* line 315 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 315 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP1);
  }
   return;

;
}

static void CodeExpr
# if defined __STDC__ | defined __cplusplus
(register tTree yyP2)
# else
(yyP2)
 register tTree yyP2;
# endif
{
/* line 321 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
 int lpindent, valindent;
  if (Tree_IsType (yyP2, kStringElement)) {
/* line 323 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 323 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   if (! ((IsArrayType (ExprType (yyP2))))) goto yyL1;
  {
/* line 323 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   if (! ((IsConstExpr (yyP2)))) goto yyL1;
  {
/* line 323 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("\"");
/* line 323 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteOccStr ((char*) (yyP2 -> StringElement . CA), CalcTypeSize (ExprType (yyP2)));
/* line 323 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("\"");
  }
  }
  }
   return;
yyL1:;

  }
  if (yyP2->Kind == kElementalExpr) {
/* line 324 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 324 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   if (! ((IsArrayType (ExprType (yyP2->ElementalExpr.Elemental))))) goto yyL2;
  {
/* line 324 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   if (! ((IsConstExpr (yyP2)))) goto yyL2;
  {
/* line 324 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("[");
/* line 324 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeConstArray (ExprType (yyP2->ElementalExpr.Elemental), yyP2->ElementalExpr.Elemental -> Elemental . CA);
/* line 324 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("]");
  }
  }
  }
   return;
yyL2:;

  }

  switch (yyP2->Kind) {
  case kStringSubscript:
/* line 325 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 325 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   if (! ((IsConstExpr (yyP2)))) goto yyL3;
  {
/* line 325 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeConst (ExprType (yyP2), ValConstExpr (yyP2));
  }
  }
   return;
yyL3:;

/* line 328 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 328 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->StringSubscript.Next);
/* line 328 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("[");
/* line 328 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->StringSubscript.Expr);
/* line 328 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("]");
  }
   return;

  case kTableSubscript:
/* line 326 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 326 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   if (! ((IsConstExpr (yyP2)))) goto yyL4;
  {
/* line 326 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeConst (ExprType (yyP2), ValConstExpr (yyP2));
  }
  }
   return;
yyL4:;

/* line 332 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 332 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->TableSubscript.Next);
/* line 332 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("[");
/* line 332 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->TableSubscript.Expr);
/* line 332 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("]");
  }
   return;

  case kStringRange:
/* line 329 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 329 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("[");
/* line 329 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->StringRange.Next);
/* line 329 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" FROM ");
/* line 329 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->StringRange.From);
/* line 329 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" FOR ");
/* line 329 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->StringRange.For);
/* line 329 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("]");
  }
   return;

  case kTable:
/* line 331 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 331 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("[");
/* line 331 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP2->Table.ExprList);
/* line 331 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("]");
  }
   return;

  case kTableRange:
/* line 333 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 333 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("[");
/* line 333 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->TableRange.Next);
/* line 333 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" FROM ");
/* line 333 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->TableRange.From);
/* line 333 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" FOR ");
/* line 333 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->TableRange.For);
/* line 333 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("]");
  }
   return;

  case kIdentifier:
/* line 335 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 335 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP2->Identifier.Sym);
  }
   return;

  case kSubscript:
/* line 336 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 336 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->Subscript.Next);
/* line 336 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("[");
/* line 336 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->Subscript.Expr);
/* line 336 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("]");
  }
   return;

  case kRange:
/* line 337 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 337 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("[");
/* line 337 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->Range.Next);
/* line 337 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" FROM ");
/* line 337 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->Range.From);
/* line 337 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" FOR ");
/* line 337 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeExpr (yyP2->Range.For);
/* line 337 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("]");
  }
   return;

  case kConvert:
/* line 339 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 339 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(");
/* line 339 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP2->Convert.Type);
/* line 339 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP2->Convert.Expr);
/* line 339 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (")");
  }
   return;

  case kExpr:
  case kBinary:
  case kUnary:
  case kConst:
  case kValof:
  case kFuncCall:
  case kElementalExpr:
  case kDefault:
  case kRound:
  case kTrunc:
  case kMostPos:
  case kMostNeg:
  case kCountArray:
  case kUnBound:
  case kStringElement:
  case kString:
/* line 341 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 341 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   if (! ((IsConstExpr (yyP2)))) goto yyL14;
  {
/* line 341 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeConst (ExprType (yyP2), ValConstExpr (yyP2));
  }
  }
   return;
yyL14:;

  break;
  }


  switch (yyP2->Kind) {
  case kValof:
/* line 343 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 343 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   SetIndent (lpindent);
/* line 343 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("( ");
/* line 343 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   SetIndent (valindent);
/* line 343 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP2->Valof.SpecList);
/* line 343 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("VALOF");
/* line 343 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Indent ();
/* line 343 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP2->Valof.Process);
/* line 343 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("RESULT ");
/* line 343 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP2->Valof.ExprList);
/* line 343 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Outdent ();
/* line 343 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   ClearIndent (valindent);
/* line 343 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (")");
/* line 343 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   ClearIndent (lpindent);
  }
   return;

  case kBinary:
/* line 344 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 344 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(");
/* line 344 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP2->Binary.Lop);
/* line 344 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" ");
/* line 344 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeOp (yyP2->Binary.Op);
/* line 344 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" ");
/* line 344 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP2->Binary.Rop);
/* line 344 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (")");
  }
   return;

  case kUnary:
/* line 345 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 345 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(");
/* line 345 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeOp (yyP2->Unary.Op);
/* line 345 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" ");
/* line 345 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP2->Unary.Expr);
/* line 345 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (")");
  }
   return;

  case kFuncCall:
/* line 346 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 346 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteSym (yyP2->FuncCall.Sym);
/* line 346 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(");
/* line 346 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP2->FuncCall.ExprList);
/* line 346 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (")");
  }
   return;

  case kDefault:
/* line 347 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 347 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("ELSE");
  }
   return;

  case kElementalExpr:
/* line 348 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 348 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP2->ElementalExpr.Elemental);
  }
   return;

  case kRound:
/* line 349 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 349 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(");
/* line 349 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP2->Round.Type);
/* line 349 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("ROUND ");
/* line 349 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP2->Round.Expr);
/* line 349 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (")");
  }
   return;

  case kTrunc:
/* line 350 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 350 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(");
/* line 350 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP2->Trunc.Type);
/* line 350 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("TRUNC ");
/* line 350 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP2->Trunc.Expr);
/* line 350 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (")");
  }
   return;

  case kCountArray:
/* line 351 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 351 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP2->CountArray.Length);
/* line 351 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (":: ");
/* line 351 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP2->CountArray.Expr);
  }
   return;

  case kUnBound:
/* line 352 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   return;

  case kNoExpr:
/* line 354 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   return;

  }

/* line 355 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 355 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   fprintf (stderr, "Fell Through CodeExpr with -\n");
/* line 355 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteTreeNode (stderr, yyP2);
/* line 355 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

;
}

static void CodeConst
# if defined __STDC__ | defined __cplusplus
(register tTree yyP4, tValue yyP3)
# else
(yyP4, yyP3)
 register tTree yyP4;
 tValue yyP3;
# endif
{

  switch (yyP4->Kind) {
  case kInt64:
/* line 361 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 361 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WritetInt (yyP3 . Number, true);
/* line 361 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(INT64)");
  }
   return;

  case kInt:
/* line 362 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 362 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WritetInt (yyP3 . Number, false);
  }
   return;

  case kInt16:
/* line 363 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 363 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WritetInt (yyP3 . Number, false);
/* line 363 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(INT16)");
  }
   return;

  case kInt32:
/* line 364 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 364 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WritetInt (yyP3 . Number, false);
/* line 364 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(INT32)");
  }
   return;

  case kReal32:
/* line 365 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 365 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteReal (yyP3);
/* line 365 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(REAL32)");
  }
   return;

  case kReal64:
/* line 366 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 366 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteReal (yyP3);
/* line 366 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("(REAL64)");
  }
   return;

  case kByte:
/* line 367 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 367 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteByte (yyP3 . Number);
  }
   return;

  case kBool:
/* line 368 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 368 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteBool (yyP3 . Number);
  }
   return;

  }

/* line 370 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 370 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   fprintf (stderr, "Fell Through CodeConst with -\n");
/* line 370 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   WriteTreeNode (stderr, yyP4);
/* line 370 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   NewLine ();
  }
   return;

;
}

static void CodeBy
# if defined __STDC__ | defined __cplusplus
(register tTree yyP5)
# else
(yyP5)
 register tTree yyP5;
# endif
{
  if (yyP5->Kind == kByVal) {
/* line 375 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 375 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("VAL ");
  }
   return;

  }
  if (yyP5->Kind == kByRef) {
/* line 376 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   return;

  }
;
}

static void CodeOp
# if defined __STDC__ | defined __cplusplus
(register tTree yyP6)
# else
(yyP6)
 register tTree yyP6;
# endif
{

  switch (yyP6->Kind) {
  case kLess:
/* line 380 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 380 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("<");
  }
   return;

  case kGreater:
/* line 381 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 381 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (">");
  }
   return;

  case kLessEqual:
/* line 382 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 382 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("<=");
  }
   return;

  case kGreaterEqual:
/* line 383 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 383 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (">=");
  }
   return;

  case kEqual:
/* line 384 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 384 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("=");
  }
   return;

  case kNotEqual:
/* line 385 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 385 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("<>");
  }
   return;

  case kAdd:
/* line 386 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 386 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("+");
  }
   return;

  case kSubtract:
/* line 387 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 387 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("-");
  }
   return;

  case kMultiply:
/* line 388 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 388 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("*");
  }
   return;

  case kDivide:
/* line 389 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 389 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("/");
  }
   return;

  case kRem:
/* line 390 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 390 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("\\");
  }
   return;

  case kBitAnd:
/* line 391 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 391 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("/\\");
  }
   return;

  case kBitOr:
/* line 392 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 392 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("\\/");
  }
   return;

  case kBitEor:
/* line 393 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 393 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("><");
  }
   return;

  case kBitNot:
/* line 394 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 394 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("~");
  }
   return;

  case kLeftShift:
/* line 395 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 395 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("<<");
  }
   return;

  case kRightShift:
/* line 396 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 396 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (">>");
  }
   return;

  case kSize:
/* line 397 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 397 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("SIZE ");
  }
   return;

  case kPlus:
/* line 398 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 398 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" PLUS ");
  }
   return;

  case kMinus:
/* line 399 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 399 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" MINUS ");
  }
   return;

  case kTimes:
/* line 400 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 400 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" TIMES ");
  }
   return;

  case kAfterOp:
/* line 401 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 401 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" AFTER ");
  }
   return;

  case kAnd:
/* line 402 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 402 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" AND ");
  }
   return;

  case kOr:
/* line 403 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 403 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" OR ");
  }
   return;

  case kNot:
/* line 404 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 404 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (" NOT ");
  }
   return;

  }

;
}

void CodeType
# if defined __STDC__ | defined __cplusplus
(register tTree yyP7)
# else
(yyP7)
 register tTree yyP7;
# endif
{
  if (yyP7 == NULL) {
/* line 408 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 408 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("NoType");
  }
   return;

  }

  switch (yyP7->Kind) {
  case kAType:
  if (yyP7->AType.Next->Kind == kNoType) {
/* line 409 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 409 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP7->AType.Type);
  }
   return;

  }
/* line 410 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 410 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP7->AType.Type);
/* line 410 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite (", ");
/* line 410 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP7->AType.Next);
  }
   return;

  case kInt:
/* line 411 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 411 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("INT ");
  }
   return;

  case kInt16:
/* line 412 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 412 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("INT16 ");
  }
   return;

  case kInt32:
/* line 413 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 413 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("INT32 ");
  }
   return;

  case kInt64:
/* line 414 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 414 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("INT64 ");
  }
   return;

  case kReal32:
/* line 415 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 415 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("REAL32 ");
  }
   return;

  case kReal64:
/* line 416 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 416 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("REAL64 ");
  }
   return;

  case kBool:
/* line 417 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 417 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("BOOL ");
  }
   return;

  case kByte:
/* line 418 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 418 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("BYTE ");
  }
   return;

  case kArray:
/* line 419 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 419 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("[");
/* line 419 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP7->Array.Expr);
/* line 419 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("]");
/* line 419 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   CodeType (yyP7->Array.Type);
  }
   return;

  case kChan:
/* line 420 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 420 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("CHAN OF ");
/* line 420 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP7->Chan.Protocol);
  }
   return;

  case kTimer:
/* line 421 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 421 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("TIMER ");
  }
   return;

  }

;
}

static void CodeConstArray
# if defined __STDC__ | defined __cplusplus
(register tTree yyP9, tConstArray yyP8)
# else
(yyP9, yyP8)
 register tTree yyP9;
 tConstArray yyP8;
# endif
{
  if (yyP9->Kind == kArray) {
  if (yyP9->Array.Type->Kind == kArray) {
/* line 425 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 426 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */

      { int i,sz;
        sz = CalcTypeSize(yyP9->Array.Type);
        for(i = 0; i < ValConstInt(yyP9->Array.Expr); ++i)
        {
          if (i) WriteChar(',');
          WriteStr("[");
          CodeConstArray(yyP9->Array.Type,yyP8+i*sz);
          WriteStr("]");
        }
      }
    
  }
   return;

  }
/* line 440 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 441 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */

      { int i;
        int sz = CalcTypeSize(yyP9->Array.Type);
        
        for(i = 0; i < ValConstInt(yyP9->Array.Expr); ++i)
        {
          if (i) WriteChar(',');
          CodeConst(yyP9->Array.Type,ReadConstArray(yyP9->Array.Type,yyP8+(sz*i)));
        }
      }
    
  }
   return;

  }
;
}

static void SemiSep
# if defined __STDC__ | defined __cplusplus
(register tTree yyP10)
# else
(yyP10)
 register tTree yyP10;
# endif
{
  if (yyP10->Kind == kAnExpr) {
  if (yyP10->AnExpr.Next->Kind == kNoExpr) {
/* line 457 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 457 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP10->AnExpr.Expr);
  }
   return;

  }
/* line 458 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 458 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   Code (yyP10->AnExpr.Expr);
/* line 458 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyWrite ("; ");
/* line 458 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   SemiSep (yyP10->AnExpr.Next);
  }
   return;

  }
/* line 460 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
  {
/* line 460 "/home/ringwood/smw/occam/source/gmd/occam.trafo" */
   yyAbort ("SemiSep");
  }
   return;

;
}

void BeginOccamOut ()
{
}

void CloseOccamOut ()
{
}
